module Paper_Wombat_MHD

importall PGFPlots
importall Images
importall FitsView

export makePDF

const path = "/Users/jdonnert/Volumes/OwnCloud/Projects/2016/WOMBAT_MHD/code_paper1/julia/"
const srcdir = "/Users/jdonnert/Volumes/OwnCloud/Projects/2016/WOMBAT_MHD/code_paper1/"

# KNL = Reds, Br1, Interlagos = Blues, Br2, BDW = Greens, Br3

function makePDF()
	
	 BlueWaters(path*"BlueWatersWeakScaling.pdf") 
	 @spawn( KNLImbalance(path*"knl_imbalance.pdf") )
	 @spawn( KNLThreadHotImbalance(path*"knl_thread_hot_cold_comparison.pdf") )
	 @spawn( StrongScaling(path*"strong_scaling.pdf") )
	 @spawn( WeakScaling(path*"weak_scaling_il.pdf", 1) )
	 @spawn( WeakScaling(path*"weak_scaling_bdw.pdf", 2) )
	 @spawn( WeakScaling(path*"weak_scaling_knl.pdf", 3) )
	 @spawn( PatchOpt(path*"patch_optimization_il.pdf",1) )
	 @spawn( PatchOpt(path*"patch_optimization_bdw.pdf",2) )
	 @spawn( PatchOpt(path*"patch_optimization_knl.pdf",3) )
	 @spawn( ThreadStrongScaling(path*"thread_strong_scaling.pdf"))
	 @spawn( KNL_VL_Scaling(path*"knl_vl_scaling.pdf") )

	 @spawn( D1_Hydro_Convergence(path*"1D_Hydro_Convergence.pdf"))
	 @spawn( D1_MHD_Convergence(path*"1D_MHD_Convergence.pdf") )
	 @spawn( D1_RJ95_2a(path*"1D_RJ95_2a.pdf"))
	 @spawn( D1_RJ95_4a(path*"1D_RJ95_4a.pdf"))

	 @spawn( D1_BrioWu(path*"1D_BriowWu.pdf") ) # expensive to run

 	 @spawn( D2_OT(path*"2D_OT_img.pdf",path*"2D_OT_slice.pdf" ) )
 	 @spawn( D2_Rotor(path*"2D_Rotor_img.pdf", path*"2D_Rotor_slice.pdf") )

	 @spawn( CPAWave(path*"CPAWaveConv.pdf", path*"CPAWaveTransv.pdf") )

	 @spawn( D3_RJ95_2a(path*"3D_RJ95_2a.pdf") )
	 @spawn( D3_RJ95_4a(path*"3D_RJ95_4a.pdf") )
	
	 @spawn( D3_BlastWave(path*"3D_BlastWave.pdf") )

	 @spawn( D3_Convergence(path*"3D_MHDConvergence.pdf") )

	 D2_BLoop(path*"2D_BLoop.pdf")
	D3_BLoop(path*"3D_BLoop.pdf")
end

function BlueWaters(fout::String)
	
	bw_weak_scaling = [
	[1,3,9,27,64,125,216,343,512,729,1000,1728,2744,4096,5832,8000,10648,16224],
    [1,3,9,27,64,125,216,343,512,729,1000,1728,2744,4096,5832],
	[2.181,2.335,2.919,2.513,2.666,2.678,2.676,2.662,2.735,3.046, 
  					2.728,2.81,2.854,3.037,3.000,3.117,3.38,3.072],
	[2.1513,2.1752,2.2031,2.2747,2.3931,2.5205,2.8396,2.9370,2.7329,2.5234,
  		2.6199,2.9822,2.7903,2.6877,3.3498] ]

	l1 = Plots.Linear(bw_weak_scaling[1], bw_weak_scaling[3]; 
				   		style="Blues9,solid,", mark="square",
						legendentry=L"$\rm1PPN - 4x4x2\, x\, 48^3$" )

 	l2 = Plots.Linear(bw_weak_scaling[2], bw_weak_scaling[4]; 
				   		style="Blues5,solid,", mark="diamond",
						legendentry=L"$\rm4PPN - 4x4x4\,x\,40^3$" )

	l3 = Plots.Linear(bw_weak_scaling[1] .*16, bw_weak_scaling[3] .*0;
				   onlyMarks=true, markSize=0)

	ax = Axis([l2,l1], 
		   	xlabel=L"Number of Nodes$$", ylabel=L"Time per Update [s] $$", 
			xmode="log",xmin=1, xmax=3e4, ymin=2.1, ymax=3.4, 
			legendPos="north west",
			style="axis x line*=left")

	ax2 = Axis([l3],
			xlabel=L"Number of Threads$$", xmin=16, xmax=16*16224, xmode="log",
			style="axis x line*=right, axis y line=none,grid=none")

	save(fout, [ax,ax2])

end

function KNLImbalance(fout::String)

	const knl_imbalance = [ [0.3,1.3],
                 			[5454935.5,2981921.1], # imbalanced 5x4x2 problem
			                [5900452.6,4686101.6]]

	l1 = Plots.Linear(knl_imbalance[1] , knl_imbalance[2]./1e6 ,style="ybar, fill=Reds9",
				   legendentry=L"\rm Imbalanced\,\, 5x2x4\,x\,48^3")
	l2 = Plots.Linear(knl_imbalance[1] , knl_imbalance[3]./1e6 ,style="ybar, fill=Reds5",
				   legendentry=L"\rm Balanced\,\, 4x2x4\,x\,48^3")

	ax = Axis([l1,l2]; 
		   xlabel="OpenMP Schedule", ylabel=L"\rm 10^6\quad Zones/sec/node",
		   ymin=0, ymax=8, 
		   style="ybar, xtick=data, enlarge x limits=0.45, bar width=35pt, 
				symbolic x coords={0.3,1.3}, xticklabels={Guided,Static},nodes near coords align={vertical}")

	save(fout, ax)
end

function KNLThreadHotImbalance(fout::AbstractString)
	
	const knl_thot_cold_weak_scaling = [[1,3,9,27,64,125],
                    					[2.1229,2.1439,2.1571,2.2828,2.2908,2.2764],
                    					[2.268,2.374,2.392,2.672,2.681,2.744] ]

	l1 = Plots.Linear(knl_thot_cold_weak_scaling[1], knl_thot_cold_weak_scaling[2]; 
				   		style="Reds8,solid,", mark="square",
						legendentry=L"KNL, Future Cray MPICH$$" )

	p1 =
	Plots.Scatter(knl_thot_cold_weak_scaling[1],knl_thot_cold_weak_scaling[2],
			   ["a","a","a","a","a","a"];
			   scatterClasses="{a={mark=*,Reds1}}")

	l2 = Plots.Linear(knl_thot_cold_weak_scaling[1], knl_thot_cold_weak_scaling[3]; 
				   		style="Reds6,solid", mark="diamond", 
						legendentry=L"KNL, Cray MPICH 7.3.1$$")

	p2 =
	Plots.Scatter(knl_thot_cold_weak_scaling[1],knl_thot_cold_weak_scaling[3],
			   ["a","a","a","a","a","a"];
			   scatterClasses="{a={mark=*,Reds2}}")

	ax = Axis([l1,l2], 
		   	xlabel=L"Number of Nodes$$", ylabel=L"Time per Update [s] $$", 
			xmode="log",xmin=1, xmax=128, ymin=2,ymax=2.8, 
			legendPos="north west")

	save(fout, ax)
end

function StrongScaling(fout::AbstractString)

	const il_strong_scaling = [	[1,2,4,28,112.0],
                     		[50.51,21.79,11.43,1.73,0.53],
                     		[50.51,25.26,12.63,1.80,0.45] ]

	const bdw_strong_scaling = [ [1,2,4,8,16,32,64,192,384],
						     [44.372,22.614,11.319,5.946,3.188,1.771,0.899,0.307,0.188],
							 [44.372,22.186,11.093,5.546,2.773,1.386,0.693,0.231,0.116]]

	const knl_strong_scaling = [[1,2,4,8,16,32.0,192,384],
    						[62.305,32.593,16.565,8.438,4.492,2.493,0.486,0.323],
							[62.305,31.153,15.576,7.788,3.894,1.947,0.324,0.162]]

	const nZones = [560*320*480.0, 1536*384.0^2, 2*640.0^2*480]

	eff = il_strong_scaling[2]/nZones[1]./il_strong_scaling[1]

	l1 =  Plots.Linear( il_strong_scaling[1], 
						il_strong_scaling[2]/nZones[1]; 
				   		style="Set12, solid", mark="square", 
						legendentry=L"Interlagos (XE6)$$")
	
	l2 =  Plots.Linear( bdw_strong_scaling[1], 
						bdw_strong_scaling[2]/nZones[2]; 
				   		style="Set13,solid", mark="diamond", 
						legendentry=L"Broadwell (XC40)$$")

	l3 =  Plots.Linear( knl_strong_scaling[1], 
						knl_strong_scaling[2]/nZones[3]; 
				   		style="Set11,solid", mark="x", 
						legendentry=L"KNL (XC40)$$")

	l4 =  Plots.Linear( bdw_strong_scaling[1], 
						bdw_strong_scaling[3]/nZones[2]; 
				   		style="black, dotted", markSize=0, 
						legendentry=L"ideal scaling$$")
	l5 =  Plots.Linear( il_strong_scaling[1], 
						il_strong_scaling[3]/nZones[1]; 
				   		style="black, dotted", markSize=0)
	l6 =  Plots.Linear( knl_strong_scaling[1], 
						knl_strong_scaling[3]/nZones[3]; 
				   		style="black,dotted", markSize=0)

	ax = Axis([l1,l2, l3, l4, l5,l6], 
		   xlabel=L"Number of Nodes$$", ylabel=L"Time/Zone [sec]$$", 
			xmin=1, xmax=512, ymin=5e-10,ymax=1e-6, ymode="log", xmode="log",
			legendPos="south west")

	save(fout, ax)
end

function WeakScaling(fout::AbstractString, i::Integer)
	
	const bw_weak_scaling = [
	[1,3,9,27,64,125,216,343,512,729,1000,1728,2744,4096,5832,8000,10648,16224],
	[2.181,2.335,2.919,2.514,2.666,2.679,2.676,2.663,2.735,3.0464,2.728,2.812, 
  	 2.855,3.0374,3.001,3.117,3.383,3.072]]
 
	const il_weak_scaling = [[1,3,9,27,64,125,150],
                   		 [2.457,2.5309,2.5314,2.9146,2.9448,2.9645,2.9645], # 1 PPN
                  		 [2.3391,2.3563,2.3977,2.439,2.4245,2.4401,2.4515], # 2 PPN
                   		 [2.0968,2.1492,2.1789,2.2313,2.2264,2.2453,2.2501], # 4 PPN
                   		[2.1237,2.1768,2.2057,2.2122,2.2008,2.2334,2.2159]] # 16 PPN
                   

	const bdw_weak_scaling = [[1,3,9,27,64,125,216,343,512],
    	[2.1398,2.2092,2.2399,2.4722,2.4794,2.4907,2.5149,2.5086,2.4987], # 1 PPN
	    [1.9567,1.9815,2.0668,2.1619,2.1831,2.1957,2.2022,2.2027,2.2055], # 2 PPN
    	[1.9925,2.0228,2.1476,2.2269,2.2452,2.2830,2.2652,2.2690,2.2626], # 4 PPN
		[1.9620,1.9712,2.0255,2.0979,2.2030,2.1432,2.1466,2.1854,2.1427]] # 36 PPN
    

	const knl_weak_scaling = [[1,3,9,27,64,125,216,343,512,648],
    	[1,3,9,27,64,125,216,343,512], # needed for 64 PPN
	    [2.0667,2.7532,2.8417,2.8224,2.8399,2.8559,2.8646,2.8754,2.842,2.8879], # 1 PPN
    	[2.1337,2.1093,2.7198,2.7917,2.7756,2.7924,2.8006,2.819,2.8147,2.828], # 2 PPN
	    [2.1229,2.1439,2.1571,2.2828,2.2908,2.2764,2.3008,2.3065,2.2828,2.3616], # 4 PPN
    	[2.1216,2.208,2.2348,2.2916,2.2721,2.3236,2.3438,2.3465,2.3289,2.3723], # 8 PPN
		[2.3035,2.3103,2.4294,2.52,2.3549,2.5395,2.3768,2.6819,2.5999]] # 64 PPN

	l1 = Plots.Linear(il_weak_scaling[1], il_weak_scaling[2], 
				   	style="Blues4, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(il_weak_scaling[1], il_weak_scaling[3], 
				   	style="Blues5, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(il_weak_scaling[1], il_weak_scaling[4], 
				   	style="Blues7, solid", mark="x", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(il_weak_scaling[1], il_weak_scaling[5], 
				   	style="Blues9, solid", mark="o", legendentry=L"16PPN$$" )
	ax1 = Axis([l1,l2, l3, l4],
		   xlabel=L"Number of Nodes$$", ylabel=L"Time/Update [sec]$$", 
			xmin=1, xmax=1024, ymin=2.0,ymax=3,  xmode="log",
			legendPos="north west")

	l1 = Plots.Linear(bdw_weak_scaling[1], bdw_weak_scaling[2], 
				   	style="Greens4, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(bdw_weak_scaling[1], bdw_weak_scaling[3], 
				   	style="Greens5, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(bdw_weak_scaling[1], bdw_weak_scaling[4], 
				   	style="Greens7, solid", mark="x", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(bdw_weak_scaling[1], bdw_weak_scaling[5], 
				   	style="Greens9, solid", mark="o", legendentry=L"36PPN$$" )

	ax2 = Axis([l1,l2, l3, l4], 
		    xlabel=L"Number of Nodes$$", ylabel=L"Time/Update [sec]$$", 
			xmin=1, xmax=1024, ymin=1.9,ymax=2.55,  xmode="log",
			legendPos="north west")

	l1 = Plots.Linear(knl_weak_scaling[1], knl_weak_scaling[3], 
				   	style="Reds3, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(knl_weak_scaling[1], knl_weak_scaling[4], 
				   	style="Reds4, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(knl_weak_scaling[1], knl_weak_scaling[5], 
				   	style="Reds5, solid", mark="x", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(knl_weak_scaling[1], knl_weak_scaling[6], 
				   	style="Reds7, solid", mark="o", legendentry=L"8PPN$$" )
	l5 = Plots.Linear(knl_weak_scaling[2], knl_weak_scaling[7], 
				   	style="Reds9, solid", mark="+", legendentry=L"64PPN$$" )

	ax3 = Axis([l1,l2, l3, l4, l5], 
		   xlabel=L"Number of Nodes$$", ylabel=L"Time/Update [sec]$$", 
			xmin=1, xmax=1024, ymin=2,ymax=2.95,  xmode="log",
			legendPos="south east", style="legend columns={2}")
	
	if i == 1 
		save(fout, ax1)
	elseif i == 2
		save(fout, ax2)
	elseif i == 3
		save(fout, ax3)
	end

	save(fout, g)
end

function PatchOpt(fout::AbstractString, i::Integer)

	const knl_patch_size = [ # patch size optimization at 27 nodes all expressed as zones/sec/node
    [16,32,40,48,50,56,60,64,72,96],
    [32,40,48,50,56,60,64,72], # needed for 16 PPN
    #[32,40,48,50,56,50,64], # needed for 64 PPN
	[32,40,48,50,56,60,64],
    [2154280.3,4956938.1,5744050.0,6022837.4,5950673.1,5846740.9,5511412.3,5470771.8,5184450.0,4377510.9], # 1 PPN
    [2285585.8,5165870.2,5815467.6,6055946.9,6086427.2,5877560.5,5758320.4,5589796.7,5253834.4,5920751.1], # 2 PPN
    [2340087.7,5131228.2,5666403.1,5767979.7,6091989.0,6009744.4,5728196.5,5595825.0,6043355.1,5917595.4], # 4 PPN
    [2320904.3,4904131.6,5528731.1,6032012.4,5648065.0,6435546.1,6247076.7,6232249.6,6108258.5,6077135.6], # 8 PPN
    [5025570.6,5833630.0,6241730.2,6593898.2,6473949.6,6422523.4,6322515.3,6151119.3], # 16 PPN
	[5197164.2,5621409.6,6188046.8,6115206.1,6364988.5,6204250.1,6124859.8] ] # 64 PPN
    

	const bdw_patch_size = [
    [16,32,40,48,50,56,60,64,72,96],
    [32,40,48,50,56,60,64,72], # needed for 12 PPN and 36 PPN
    [16,32,40,48,50,56,60,64,72], # needed for 18 PPN
    [2549671.4,4190193.1,3946991.6,3271885.4,3286179.2,2976822.6,2928537.8,2762911.2,2709303.8,2446650.4], # 1 PPN
    [2832665.9,4636576.1,4436985.8,3681116.9,3544702.6,3509902.0,3480518.3,3286080.1,3296758.4,3222462.3], # 2 PPN
    [2976681.8,4602872.3,4379633.4,3702814.0,3607586.2,3490562.16,3354362.8,3160194.6,3291146.4,3248194.5], # 4 PPN
    [4635428.9,4448673.9,3844217.5,3769069.6,3428288.8,3321373.6,3317967.3,3320506.5],  # 12 PPN
    [3087185.8,4454801.7,4356046.2,3853706.1,3686323.1,3454136.9,3516682.3,3192084.1,3286196.2], # 18 PPN
    [4794523.0,4522714.5,3871365.2,3713221.4,3423795.8,3377492.0,3341108.6,3310525.8] # 36 PPN
    ]


	const il_patch_size = [# 2.5 GHz AMD Opteron(tm) Processor 6281
    [16,32,40,48,50,56,60,64,72,96],
    [16,32,40,48,50,56,60,64,72], # needed for 16 PPN
    [746415.8,1589572.1,1636454.9,1541889.1,1491901.1,1404506.6,1195537.4,1072894.9,886574.8,985160.8], # 1PPN
    [998573.3,1663291.6,1775114.2,1639937.4,1719331.4,1446768.5,1187832.9,1558043.1,1068139.5,1303118.4], # 2 PPN
    [1069894.6,1683728.1,1935022.3,1915142.5,1884937.9,1782089.4,1611470.5,1707384.5,1666169.4,1619542.8], # 4 PPN
    [1025706.5,1608245.3,1961766.3,1911444.5,1748142.5,1701066.7,1616293.8,1714106.3,1664196.1,1422476.6], # 8 PPN
	[1103592.1,1812635.3,1976615.9,1938403.8,1915499.1,1778817.6,1615916.0,1724584.3,1669056.6]] # 16 PPN
	
	l1 = Plots.Linear(il_patch_size[1], il_patch_size[3]/1e6, 
				   	style="Blues5, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(il_patch_size[1], il_patch_size[4]/1e6, 
				   	style="Blues6, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(il_patch_size[1], il_patch_size[5]/1e6, 
				   	style="Blues7, solid", mark="triangle", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(il_patch_size[1], il_patch_size[6]/1e6, 
				   	style="Blues8, solid", mark="o", legendentry=L"8PPN$$" )
	l5 = Plots.Linear(il_patch_size[2], il_patch_size[7]/1e6, 
				   	style="Blues9, solid", mark="star", legendentry=L"16PPN$$" )

	ax1 = Axis([l1,l2, l3, l4,l5],
		xlabel=L"Cubic Patch Size $\rm N^3$",	ylabel=L"$10^6$ Zones/Second/Node", 
			xmin=8, xmax=100, ymin=0.6,ymax=2.3,
			legendPos="north east", style="legend columns={2}")

	l1 = Plots.Linear(bdw_patch_size[1], bdw_patch_size[4]/1e6, 
				   	style="Greens4, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(bdw_patch_size[1], bdw_patch_size[5]/1e6, 
				   	style="Greens5, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(bdw_patch_size[1], bdw_patch_size[6]/1e6, 
				   	style="Greens6, solid", mark="triangle", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(bdw_patch_size[2], bdw_patch_size[7]/1e6, 
				   	style="Greens7, solid", mark="o", legendentry=L"12PPN$$" )
	l5 = Plots.Linear(bdw_patch_size[3], bdw_patch_size[8]/1e6, 
				   	style="Greens8, solid", mark="star", legendentry=L"18PPN$$" )
	l6 = Plots.Linear(bdw_patch_size[2], bdw_patch_size[9]/1e6, 
				   	style="Greens9, solid", mark="+", legendentry=L"36PPN$$" )

	ax2 = Axis([l1,l2, l3, l4, l5, l6], 
		xlabel=L"Cubic Patch Size $\rm N^3$",	ylabel=L"$10^6$  Zones/Second/Node", 
			xmin=8, xmax=100, ymin=2.3,ymax=4.9,
			legendPos="north east", style="legend columns={2}")

	l1 = Plots.Linear(knl_patch_size[1], knl_patch_size[4]/1e6, 
				   	style="Reds4, solid", mark="square", legendentry=L"1PPN$$" )
	l2 = Plots.Linear(knl_patch_size[1], knl_patch_size[5]/1e6, 
				   	style="Reds5, solid", mark="diamond", legendentry=L"2PPN$$" )
	l3 = Plots.Linear(knl_patch_size[1], knl_patch_size[6]/1e6, 
				   	style="Reds6, solid", mark="triangle", legendentry=L"4PPN$$" )
	l4 = Plots.Linear(knl_patch_size[1], knl_patch_size[7]/1e6, 
				   	style="Reds7, solid", mark="o", legendentry=L"8PPN$$" )
	l5 = Plots.Linear(knl_patch_size[2], knl_patch_size[8]/1e6, 
				   	style="Reds8, solid", mark="star", legendentry=L"16PPN$$" )
	l6 = Plots.Linear(knl_patch_size[3], knl_patch_size[9]/1e6, 
				   	style="Reds9, solid", mark="+", legendentry=L"64PPN$$" )

	ax3 = Axis([l1, l2, l3, l4, l5, l6], 
			xlabel=L"Cubic Patch Size $\rm N^3$", ylabel=L"$10^6$ Zones/Second/Node", 
			xmin=8, xmax=100, ymin=2,ymax=6.9,
			legendPos="south east", style="legend columns={2}")

	if i == 1 
		save(fout, ax1)
	elseif i == 2
		save(fout, ax2)
	elseif i == 3
		save(fout, ax3)
	end
end

function ThreadStrongScaling(fout::AbstractString)

	const il_thread_strong_scaling = [ # 8x7x4 x 40^3
    	[1,2,4,8,16],
	    [86.533,45.48,26.8435,14.6365,8.17675],
    	[86.533,43.2665,21.63325,10.816625,5.4083125] ]
    

	const bdw_thread_strong_scaling = [# 12x12x8 x 32^3
                             [1,2,4,9,18,36],
                             [135.5,72.48,39.53,21.07,15.28,8.49],
                             [135.5,67.75,33.875,15.06,7.53,3.76],
							 [177.2,91.2,46.58,22.37,15.14,8.56]]    


	const knl_thread_strong_scaling = [# 8x8x8 x 48^3
							[1,2,4,8,16,32,64,68],
							[344.2,197.57,99.18,50.38,25.9125,13.601,8.72,8.52],
							[344.2,172.1,86.05,43.025,21.5125,10.75625,5.378125,5.061] ]

	const nZones = [8*7*4*40^3, 12^2*8*32^3, 8^3 * 48^3]
	
	l1 =  Plots.Linear( il_thread_strong_scaling[1], 
						il_thread_strong_scaling[2]/nZones[1]; 
				   		style="Set12, solid", mark="square", 
						legendentry=L"Interlagos (XE6)$$")
	
	l2 =  Plots.Linear( bdw_thread_strong_scaling[1], 
						bdw_thread_strong_scaling[2]/nZones[2]; 
				   		style="Set13,solid", mark="diamond", 
						legendentry=L"Broadwell (XC40)$$")

	l7 =  Plots.Linear( bdw_thread_strong_scaling[1], 
						bdw_thread_strong_scaling[4]/nZones[2]; 
				   		style="Set13, dashed", markSize=0, 
						legendentry=L"Broadwell w/o turbo$$")

	l3 =  Plots.Linear( knl_thread_strong_scaling[1], 
						knl_thread_strong_scaling[2]/nZones[3]; 
				   		style="Set11,solid", mark="x", 
						legendentry=L"KNL (XC40)$$")

	l4 =  Plots.Linear( bdw_thread_strong_scaling[1], 
						bdw_thread_strong_scaling[3]/nZones[2]; 
				   		style="black, dotted", markSize=0, 
						legendentry=L"ideal scaling$$")
	l5 =  Plots.Linear( il_thread_strong_scaling[1], 
						il_thread_strong_scaling[3]/nZones[1]; 
				   		style="black, dotted", markSize=0)
	l6 =  Plots.Linear( knl_thread_strong_scaling[1], 
						knl_thread_strong_scaling[3]/nZones[3]; 
				   		style="black,dotted", markSize=0)

	ax = Axis([l1,l2,  l7, l3,l4, l5,l6], 
		   xlabel=L"Number of Threads$$", ylabel=L"Time/Zone [sec]$$", 
			xmin=1, xmax=64, ymin=1e-7,ymax=1e-5, ymode="log", xmode="log",
			legendPos="north east",
			style="xtick={1,2,4,8,16,32,64},xticklabels={1,2,4,8,16,32,64}")

	save(fout, ax)
end

function KNL_VL_Scaling(fout::AbstractString)

	const knl_vl_scaling = [ [64,128,256,512],# VL study (17x4x4 x 48^3 - 68 threads)
						 	 [10.93,7.73,5.54,4.51] ]

	l1 =  Plots.Linear( knl_vl_scaling[1], knl_vl_scaling[2]; 
				   		style="Set11,solid", mark="square", legendentry="KNL,
						68 threads")

	ax = Axis([l1], 
		   xlabel=L"Vector Length [bits]$$", ylabel=L"Time per Update [s] $$", 
			xmode="log",xmin=64, xmax=512, ymin=4,ymax=16, ymode="log", 
			legendPos="north west", 
			style="xticklabels={64,128,256,512},xtick={64,128,256,512},
			yticklabels={4,8,16}, ytick={4,8,16}")

	save(fout, ax)
end

function D1_MHD_Convergence(fout::String)

	files = [ "AthenaSuite_LinMHD_Alf1d_conv.data", "AthenaSuite_LinMHD_Ent1d_conv.data",  
		  	  "AthenaSuite_LinMHD_Fst1d_conv.data", "AthenaSuite_LinMHD_Slo1d_conv.data"]

	tmp = readdlm(srcdir*"Athena_data/1dMHD/"*files[1])
	l1 = Plots.Linear(tmp[:,1], tmp[:,2], mark="square", style="Dark21", legendentry="Alfven")

	tmp = readdlm(srcdir*"Athena_data/1dMHD/"*files[2])
	l2 = Plots.Linear(tmp[:,1], tmp[:,2], mark="diamond", style="Dark22", legendentry="Entropy")

	tmp = readdlm(srcdir*"Athena_data/1dMHD/"*files[3])
	l3 = Plots.Linear(tmp[:,1], tmp[:,2], mark="x", style="Dark23", legendentry="Fast mode")
	
	tmp = readdlm(srcdir*"Athena_data/1dMHD/"*files[4])
	l4 = Plots.Linear(tmp[:,1], tmp[:,2], mark="o", style="Dark24", legendentry="Slow mode")

	x = logspace(log10(8),log10(1024), 50)
	y = x.^(-2)
	y *= 3e-7/maximum(y)

	l5 = Plots.Linear(x,y; markSize=0, style="black,densely dotted", legendentry="ideal scaling")

	ax =  Axis([l1,l2,l3,l4,l5]; xmin=8,xmax=1024,ymin=1.5e-11,ymax=1e-6, xlabel="Number of Zones",
					ylabel="L1 Error", xmode="log", ymode="log")

	save(fout, ax)
end

function D1_Hydro_Convergence(fout::String)

	files = [ "AthenaSuite_LinHyd_Ent1d_conv.data", "AthenaSuite_LinHyd_Snd1d_conv.data",  
		  	  "AthenaSuite_LinHyd_Vy1d_conv.data", "AthenaSuite_LinHyd_Vz1d_conv.data"]

	tmp = readdlm(srcdir*"Athena_data/1dhydro/"*files[1])
	l1 = Plots.Linear(tmp[:,1], tmp[:,2], mark="square", style="Dark21", legendentry="Entropy")

	tmp = readdlm(srcdir*"Athena_data/1dhydro/"*files[2])
	l2 = Plots.Linear(tmp[:,1], tmp[:,2], mark="diamond", style="Dark22", legendentry="Sound")

	tmp = readdlm(srcdir*"Athena_data/1dhydro/"*files[3])
	l3 = Plots.Linear(tmp[:,1], tmp[:,2], mark="x", style="Dark23", legendentry="Y-Velocity")
	
	tmp = readdlm(srcdir*"Athena_data/1dhydro/"*files[4])
	l4 = Plots.Linear(tmp[:,1], tmp[:,2]; mark="o", style="Dark24", legendentry="Z-Velocity")

	x = logspace(log10(8),log10(1024), 50)
	y = x.^(-2)
	y *= 3e-7/maximum(y)

	l5 = Plots.Linear(x,y; markSize=0, style="black,densely dotted", legendentry="ideal scaling")

	ax =  Axis([l1,l2,l3,l4,l5]; xmin=8,xmax=1024,ymin=1.5e-11,ymax=1e-6, xlabel="Number of Zones",
					ylabel="L1 Error", xmode="log", ymode="log")

	save(fout, ax)
end

function D1_RJ95_2a(fout::String)

	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-RHO.xstrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-PRS.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-energy.xstrip")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-Vx.xstrip")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-Vy.xstrip")
	l5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-Vz.xstrip")
	l6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-By.xstrip")
	l7 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/2a/RJ950-0001-Bz.xstrip")
	l8 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, markSize=1)

	const noxt = "xticklabels={}"

	ax1 = Axis([l1], ylabel=L"Density$$",xmin=-0.5,xmax=0.5,ymin=0.95,ymax=1.65,style=noxt)
	ax2 = Axis([l2], ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.91,ymax=2.1,style=noxt)
	ax3 = Axis([l3], ylabel=L"Energy$$",xmin=-0.5,xmax=0.5,ymin=2.35,ymax=4.8,style=noxt)
	ax4 = Axis([l4], ylabel=L"x-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=1.45,style=noxt)
	ax5 = Axis([l5], ylabel=L"y-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=0.25,style=noxt)
	ax6 = Axis([l6], ylabel=L"z-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=0.61,style=noxt)
	ax7 = Axis([l7], xlabel="x", ylabel=L"y-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=0.91,
								ymax=1.7)
	ax8 = Axis([l8], xlabel="x", ylabel=L"z-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=0.41,
								ymax=0.85)

	g = GroupPlot(2, 4, groupStyle = "vertical sep = 0cm, horizontal sep = 1.8cm")

	push!(g, ax1)
	push!(g, ax2)
	push!(g, ax3)
	push!(g, ax4)
	push!(g, ax5)
	push!(g, ax6)
	push!(g, ax7)
	push!(g, ax8)
	
	save(fout, g)
end

function D1_RJ95_4a(fout::String)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-RHO.xstrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-PRS.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-energy.xstrip")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-Vx.xstrip")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-Vy.xstrip")
	l5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
											markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-Vz.xstrip")
	l6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-By.xstrip")
	l7 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/1D/4a/RJ950-0001-Bz.xstrip")
	l8 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)

	const noxt = "xticklabels={}"

	ax1 = Axis([l1], ylabel=L"Density$$",xmin=-0.5,xmax=0.5,ymin=0,ymax=1.1,style=noxt)
	ax2 = Axis([l2], ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.01,ymax=1.1,style=noxt)
	ax3 = Axis([l3], ylabel=L"Energy$$",xmin=-0.5,xmax=0.5,ymin=0.51,ymax=2.61,style=noxt)
	ax4 = Axis([l4], ylabel=L"x-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.11,ymax=1.01,style=noxt)
	ax5 = Axis([l5], ylabel=L"y-velocity$$",xmin=-0.5,xmax=0.5,ymin=-1.01,ymax=0.11,style=noxt)
	ax6 = Axis([l6], ylabel=L"z-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=0.21,style=noxt)
	ax7 = Axis([l7], xlabel="x", ylabel=L"y-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=-0.21,
					ymax=1.1)
	ax8 = Axis([l8], xlabel="x", ylabel=L"z-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=-0.21,
					ymax=0.21)

	g = GroupPlot(2, 4, groupStyle = "vertical sep = 0cm, horizontal sep = 1.8cm")

	push!(g, ax1)
	push!(g, ax2)
	push!(g, ax3)
	push!(g, ax4)
	push!(g, ax5)
	push!(g, ax6)
	push!(g, ax7)
	push!(g, ax8)
	
	save(fout, g)
end

function D1_BrioWu(fout::String)

	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-RHO.xstrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-PRS.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-Spc.xstrip")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-Vx.xstrip")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-Vy.xstrip")
	l5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriWu-0002-By.xstrip")
	l6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=1, legendentry="400 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-RHO.xstrip")
	h1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-PRS.xstrip")
	h2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-Spc.xstrip")
	h3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-Vx.xstrip")
	h4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-Vy.xstrip")
	h5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")
	
	tmp = readdlm(srcdir*"Athena_data/BrioWu/BriW2-0002-By.xstrip")
	h6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="10000 zones")

	const noxt = "xticklabels={}"

	ax1 = Axis([l1,h1]; ylabel=L"Density$$",xmin=-0.5,xmax=0.5,ymin=0.1,ymax=1.1,style=noxt)
	ax2 = Axis([l2,h2]; ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.05,ymax=1.1,style=noxt)
	ax3 = Axis([l3,h3]; ylabel=L"Energy$$",xmin=-0.5,xmax=0.5,ymin=0.55,ymax=1.95,style=noxt)
	ax4 = Axis([l4,h4]; ylabel=L"x-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.31,ymax=0.81,
						style=noxt)
	ax5 = Axis([l5,h5]; xlabel="x", ylabel=L"y-velocity$$",xmin=-0.5,xmax=0.5,ymin=-1.81,
						ymax=0.05, legendPos="south west")
	ax6 = Axis([l6,h6]; xlabel="x", ylabel=L"y-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=-1.1,
						ymax=1.1)

	g = GroupPlot(2, 3, groupStyle = "vertical sep = 0cm, horizontal sep = 1.8cm")

	push!(g, ax1)
	push!(g, ax2)
	push!(g, ax3)
	push!(g, ax4)
	push!(g, ax5)
	push!(g, ax6)

	save(fout, g)
end

function blow_up(data::Array, nOld, nNew)

	new = zeros(Float64, nNew, nNew)

	for i=1:nNew
		for j=1:nNew

			iOld = Int64(ceil(i *nOld/nNew )) 
			jOld = Int64(ceil(j * nOld/nNew ))

			new[i,j] = data[iOld, jOld]

		end
	end

	return new
end

function D2_OT(fimg::String, fplot::String)
	
	data = zeros(Float64, 1024,1024, 4)

	fp = open(srcdir*"Athena_data/OTvortex/OTvor-0002-RHO.bof", "r")
	data[:,:,1] = blow_up(read(fp, Float32, 192, 192), 192, 1024)
	close(fp)

	fp = open(srcdir*"Athena_data/OTvortex/OTvor-0002-PRS.bof", "r")
	data[:,:,2] = blow_up(read(fp, Float32, 192, 192), 192, 1024)
	close(fp)

	fp = open(srcdir*"Athena_data/OTvortex/OTvor-0002-Vsq.bof", "r")
	data[:,:,3] = blow_up(read(fp, Float32, 192, 192), 192, 1024)
	close(fp)
	
	fp = open(srcdir*"Athena_data/OTvortex/OTvor-0002-Bsq.bof", "r")
	data[:,:,4] = blow_up(read(fp, Float32, 192, 192), 192, 1024)
	close(fp)

	slice = [1,2,3,4]

	Fits_View(data; fout=fimg, slice=slice,	annoText=["Density","Pressure","v^2","B^2"],
		   			colmap=39, log=false, noColBar=true, scaleText=L"\rm 192^2\,zones",
					annoCol=["white","white","white","white"], scaleCol="white", frameCol="white")

	tmp = readdlm(srcdir*"Athena_data/OTvortex/OTvor-PRS.top.xstrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=0.5, legendentry="192 zones")
	tmp = readdlm(srcdir*"Athena_data/OTvortex/OTvo2-PRS.top.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11",   markSize=0, legendentry="512 zones")

	g = GroupPlot(1, 2, groupStyle = "vertical sep = 0cm")

	ax1 = Axis([l1,l2]; ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.01,ymax=0.31,
						style="xticklabels={}")

	tmp = readdlm(srcdir*"Athena_data/OTvortex/OTvor-PRS.btm.xstrip")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   markSize=0.5, legendentry="192 zones")
	tmp = readdlm(srcdir*"Athena_data/OTvortex/OTvo2-PRS.btm.xstrip")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0, legendentry="512 zones")

	g = GroupPlot(1, 2, groupStyle = "vertical sep = 0cm")

	ax2 = Axis([l3,l4]; ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.01,ymax=0.41)

	push!(g, ax1)
	push!(g, ax2)

	save(fplot, g)
end

function D2_Rotor(fimg::String, fplot::String)
	
	data = zeros(Float64, 1024,1024, 4)

	fp = open(srcdir*"Athena_data/MHDrotor/Rotor-0003-RHO.bof", "r")
	data[:,:,1] = blow_up(read(fp, Float32, 400, 400), 400, 1024)
	close(fp)

	fp = open(srcdir*"Athena_data/MHDrotor/Rotor-0003-PRS.bof", "r")
	data[:,:,2] = blow_up(read(fp, Float32, 400, 400), 400, 1024)
	close(fp)

	fp = open(srcdir*"Athena_data/MHDrotor/Rotor-0003-Mch.bof", "r")
	data[:,:,3] = blow_up(read(fp, Float32, 400, 400), 400, 1024)
	close(fp)
	
	fp = open(srcdir*"Athena_data/MHDrotor/Rotor-0003-Bsq.bof", "r")
	data[:,:,4] = blow_up(read(fp, Float32, 400, 400), 400, 1024)
	close(fp)

	slice = [1,2,3,4]

	Fits_View(data; fout=fimg, slice=slice,
		   			annoText=["Density","Pressure","Mach Number","B^2"],
		   			colmap=39, log=false, noColBar=true, scaleText=L"\rm 400^2\, zones",
				    annoCol=["white","white","white","white"], scaleCol="white", frameCol="white")

	tmp = readdlm(srcdir*"Athena_data/MHDrotor/Rotor-Bx.ystrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=0.5, legendentry="y-direction")

	ax1 = Axis([l1]; ylabel=L"Magnetic Field$$",xmin=-0.5,xmax=0.5,ymin=0.01,ymax=2.25,
		  			style="xticklabels={}")

	tmp = readdlm(srcdir*"Athena_data/MHDrotor/Rotor-By.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="Set11", markSize=0.5, onlyMarks=true, 
				   							legendentry="y-direction")

	ax2 = Axis([l2]; ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=-1.11,ymax=1.21 )

	g = GroupPlot(1, 2, groupStyle = "vertical sep = 0cm")

	push!(g, ax1)
	push!(g, ax2)

	save(fplot, g)
end

function CPAWave(fconv::String, ftrans::String)
	
	tmp = readdlm(srcdir * 
			   "Athena_data/CircPolarizedAlfven/AthenaSuite_CircPolAlf2d_conv.data")

	l1 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark21", mark="square",
				   							 legendentry="2D")

	tmp = readdlm(srcdir * 
			   "Athena_data/CircPolarizedAlfven/AthenaSuite_CircPolAlf3d_conv.data")

	l2 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark22", mark="diamond",
				   							legendentry="3D")
	
	x = logspace(log10(8),log10(1024), 50)
	y = x.^(-2)
	y *= 0.1/maximum(y)

	l3 = Plots.Linear(x, y; style="black,dotted", mark="square",
				   							markSize=0, legendentry="ideal")

	ax = Axis([l1,l2,l3]; ylabel=L"L1 Error$$", xlabel=L"Number of Zones$$", 
		   				xmin=8,xmax=1024,ymin=1e-5,ymax=0.1, xmode="log", ymode="log" )

	save(fconv, ax)

	tmp = readdlm(srcdir*"Athena_data/CircPolarizedAlfven/2D_Btrans/B2_CPA0008.dat")
	l1 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark21,const plot", markSize=0,
				   legendentry="N=8")

	tmp = readdlm(srcdir*"Athena_data/CircPolarizedAlfven/2D_Btrans/B2_CPA0016.dat")
	l2 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark22,const plot",markSize=0,
				   legendentry="N=16")

	tmp = readdlm(srcdir*"Athena_data/CircPolarizedAlfven/2D_Btrans/B2_CPA0032.dat")
	l3 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark23,const plot",markSize=0,
				   legendentry="N=32")
	
	tmp = readdlm(srcdir*"Athena_data/CircPolarizedAlfven/2D_Btrans/B2_CPA0064.dat")
	l4 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark24,const plot",markSize=0,
				   legendentry="N=64")

	tmp = readdlm(srcdir*"Athena_data/CircPolarizedAlfven/2D_Btrans/B2_CPA0128.dat")
	l5 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark25,const plot",markSize=0,
				   legendentry="N=128")

	ax = Axis([l5,l4,l3,l2,l1]; ylabel=L"$B_2$", xlabel=L"x$$", 
		   				xmin=-1,xmax=1,ymin=-0.1,ymax=0.1, legendPos="south west")

	save(ftrans, ax)
end

function D2_BLoop(fout::String)

	data = zeros(Float64, 1024, 1024, 2)

	fp = open(srcdir*"Athena_data/Bloop2d/B0p2D-0000-Bp.bof", "r")
	tmp = read(fp, Float32, 256, 128)
	tmp = tmp[65:192, 1:128]
	data[:,:,1] = blow_up(tmp, 128, 1024)

	fp = open(srcdir*"Athena_data/Bloop2d/B0p2D-0002-Bp.bof", "r")
	tmp = read(fp, Float32, 256, 128)
	tmp = tmp[65:192, 1:128]
	data[:,:,2] = blow_up(tmp, 128, 1024)

	Fits_View(data; fout=fout, slice=[1,2],	
				    annoText=["B^2, t=0","B^2, t=2"],zmin=0,zmax=1e-6,
		  			colmap=39, log=false, noColBar=true, 
					scaleText=L"\rm 128^2 \,\, of \,\, 256x128\,\,zones", 
				    annoCol=["white","white","white"], scaleCol="white", frameCol="white")
end

function D3_RJ95_2a(fout::String)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/RHO-cut-short.txt")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/PRS-cut-short.txt")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/energy-cut-short.txt")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/Vx1-cut-short.txt")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/Vy1-cut-short.txt")
	l5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
											markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/Vz1-cut-short.txt")
	l6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/By1-cut-short.txt")
	l7 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-oblique/2a/Bz1-cut-short.txt")
	l8 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)

	const noxt = "xticklabels={}"

	ax1 = Axis([l1], ylabel=L"Density$$",xmin=-0.5,xmax=0.5,ymin=0.95,ymax=1.65,style=noxt)
	ax2 = Axis([l2], ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.91,ymax=2.1,style=noxt)
	ax3 = Axis([l3], ylabel=L"Energy$$",xmin=-0.5,xmax=0.5,ymin=2.35,ymax=4.8,style=noxt)
	ax4 = Axis([l4], ylabel=L"x-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=1.45,style=noxt)
	ax5 = Axis([l5], ylabel=L"y-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=0.25,style=noxt)
	ax6 = Axis([l6], ylabel=L"z-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.21,ymax=0.61,style=noxt)
	ax7 = Axis([l7], xlabel="x", ylabel=L"y-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=0.91,
								ymax=1.7)
	ax8 = Axis([l8], xlabel="x", ylabel=L"z-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=0.41,
								ymax=0.85)

	g = GroupPlot(2, 4, groupStyle = "vertical sep = 0cm, horizontal sep = 1.8cm")

	push!(g, ax1)
	push!(g, ax2)
	push!(g, ax3)
	push!(g, ax4)
	push!(g, ax5)
	push!(g, ax6)
	push!(g, ax7)
	push!(g, ax8)
	
	save(fout, g)

end

function D3_RJ95_4a(fout::String)
	
		
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-RHO.xstrip")
	l1 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-PRS.xstrip")
	l2 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-energy.xstrip")
	l3 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-Vx.xstrip")
	l4 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-Vy.xstrip")
	l5 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
											markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-Vz.xstrip")
	l6 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-By.xstrip")
	l7 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true, 
				   							markSize=1)
	
	tmp = readdlm(srcdir*"data/RJ95/3D-align/4a/RJ950-0001-Bz.xstrip")
	l8 = Plots.Linear( tmp[:,1], tmp[:,2]; style="black", mark="square", onlyMarks=true,
				   							markSize=1)

	const noxt = "xticklabels={}"

	ax1 = Axis([l1], ylabel=L"Density$$",xmin=-0.5,xmax=0.5,ymin=0,ymax=1.1,style=noxt)
	ax2 = Axis([l2], ylabel=L"Pressure$$",xmin=-0.5,xmax=0.5,ymin=0.01,ymax=1.1,style=noxt)
	ax3 = Axis([l3], ylabel=L"Energy$$",xmin=-0.5,xmax=0.5,ymin=0.51,ymax=2.71,style=noxt)
	ax4 = Axis([l4], ylabel=L"x-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.11,ymax=1.01,style=noxt)
	ax5 = Axis([l5], ylabel=L"y-velocity$$",xmin=-0.5,xmax=0.5,ymin=-1.01,ymax=0.11,style=noxt)
	ax6 = Axis([l6], ylabel=L"z-velocity$$",xmin=-0.5,xmax=0.5,ymin=-0.22,ymax=0.22,style=noxt)
	ax7 = Axis([l7], xlabel="x", ylabel=L"y-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=-0.22,
					ymax=1.1)
	ax8 = Axis([l8], xlabel="x", ylabel=L"z-magnetic field$$",xmin=-0.51,xmax=0.5,ymin=-0.22,
					ymax=0.22)

	g = GroupPlot(2, 4, groupStyle = "vertical sep = 0cm, horizontal sep = 1.8cm")

	push!(g, ax1)
	push!(g, ax2)
	push!(g, ax3)
	push!(g, ax4)
	push!(g, ax5)
	push!(g, ax6)
	push!(g, ax7)
	push!(g, ax8)
	
	save(fout, g)
end

function make_data(tmp::Array)

	data = zeros(Float64,300,300)

	l = 1

	for i=1:300
		for j=1:200
	
			data[i,j+50] = tmp[l,3]

			l += 1
		end
	end

	data2 = zeros(Float64, 200, 200)

	for i=1:200
		for j=1:200

			data2[i,j] = data[50+i,50+j]

		end
	end
	
	return data2
end

function D3_BlastWave(fout::String)
	
	data = zeros(Float64, 1024, 1024, 3)

	tmp = readdlm(srcdir*"Athena_data/MagBlastWave3D/MBW3D-0002-RHO.xysof.txt")
	tmp = make_data(tmp)
	data[:,:,1] = blow_up(tmp, 200, 1024)

	tmp = readdlm(srcdir*"Athena_data/MagBlastWave3D/MBW3D-0002-vsq.xysof.txt")
	tmp = make_data(tmp)
	data[:,:,2] = blow_up(tmp, 200, 1024)

	tmp = readdlm(srcdir*"Athena_data/MagBlastWave3D/MBW3D-0002-Bsq.xysof.txt")
	tmp = make_data(tmp)
	data[:,:,3] = blow_up(tmp, 200, 1024)

	Fits_View(data; fout=fout, slice=[1,2,3],	annoText=["Density","v^2","B^2"],
		  			colmap=39, log=false, noColBar=true, scaleText=L"\rm\quad 200^2\,\,zones",
				    annoCol=["white","white","white"], scaleCol="white", frameCol="white")

end

function D3_Convergence(fout::String)

	tmp = readdlm(srcdir * 
			   "Athena_data/3dMHD_conv/AthenaSuite_LinMHD_Alf3d_conv.data")
	l1 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark21", mark="square",
				   							legendentry="Alfven Wave 3D")
	tmp = readdlm(srcdir * 
			   "Athena_data/3dMHD_conv/AthenaSuite_LinMHD_Ent3d_conv.data")
	l2 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark22", mark="diamond",
				   						legendentry="Entropy 3D")
	tmp = readdlm(srcdir * 
			   "Athena_data/3dMHD_conv/AthenaSuite_LinMHD_Fst3d_conv.data")
	l3 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark23", mark="x",
				   							legendentry="Fast mode 3D")
	
	tmp = readdlm(srcdir * 
			   "Athena_data/3dMHD_conv/AthenaSuite_LinMHD_Slo3d_conv.data")
	l4 = Plots.Linear(tmp[:,1], tmp[:,2]; style="Dark24", mark="o",
				   							legendentry="Slow mode 3D")
	
	x = logspace(log10(8),log10(1024), 50)
	y = x.^(-2)
	y *= 1e-6/maximum(y)

	l5 = Plots.Linear(x, y; style="black,dotted", mark="square",
				   							markSize=0, legendentry="ideal scaling")

	ax = Axis([l1,l2,l3,l4,l5]; ylabel=L"L1 Error$$", xlabel=L"Number of Zones$$", 
		   				xmin=8,xmax=128,ymin=5e-9,ymax=1e-6, xmode="log", ymode="log")

	save(fout, ax)

end

function D3_AWave(fout::String)
	
	save(fout, ax)
end

function D3_BLoop(fout::String)

	data = zeros(Float64, 1024,1024, 2)
	img = load(srcdir*"Athena_data/Bloop3d/Bloop3d-0.jpg")
	data[:,:,1] = Images.data(img)
	img = load(srcdir*"Athena_data/Bloop3d/Bloop3d-2.jpg")
	data[:,:,2] = Images.data(img)
	print(minimum(data))
	print(maximum(data))
	slice = [1,2]

	Fits_View(data; fout=fout, slice=slice,
		   			annoText=["B^2, t=0","B^2, t=2"], frameCol="white",zmin=0, zmax=1.0,
		   			colmap=39, log=false, noColBar=true, scaleText=L"\rm\quad\quad 400^2\, zones",
				    annoCol=["white","white","white","white"], scaleCol="white")
end


end # module
